@@ -44,23 +44,35 @@ class MessageInteraction(BaseInteraction):
             self.message = None
             self.components = []
         else:
-            components = msg_data.get("components", [])
-            self.components = [ActionRow.from_dict(comp) for comp in components]
-            # For some reason "channel_id" might not be included in message data
-            if "channel_id" in msg_data:
-                channel = client.get_channel(int(msg_data["channel_id"]))
+            if msg_data.get("flags") == 64:
+                # The original message is ephemeral
+                message_id = int(msg_data["id"])
+                self.message = None
+                self.components = []
+                for message in client.cached_ephemeral_messages:
+                    if message.id == message_id and message.channel.id == self.channel.id:
+                        self.message = message
+                        self.components = message.components
+                        break
             else:
-                msg_data["channel_id"] = self.channel_id
-                channel = self.channel
-            # For some reason "channel_id" in message reference might not be included
-            if "channel_id" not in msg_data["message_reference"]:
-                msg_data["message_reference"]["channel_id"] = None if channel is None else channel.id
-            # channel must not be None, because channel.id attr is needed in discord.Message.__init__
-            self.message = discord.Message(
-                state=state,
-                channel=channel or PartialTextChannel(0),
-                data=msg_data
-            )
+                components = msg_data.get("components", [])
+                self.components = [ActionRow.from_dict(comp) for comp in components]
+                # For some reason "channel_id" might not be included in message data
+                if "channel_id" in msg_data:
+                    channel = client.get_channel(int(msg_data["channel_id"]))
+                else:
+                    msg_data["channel_id"] = self.channel_id
+                    channel = self.channel
+                # For some reason "channel_id" in message reference might not be included
+                if "message_reference" in msg_data:
+                    if "channel_id" not in msg_data["message_reference"]:
+                        msg_data["message_reference"]["channel_id"] = None if channel is None else channel.id
+                # channel must not be None, because channel.id attr is needed in discord.Message.__init__
+                self.message = discord.Message(
+                    state=state,
+                    channel=channel or PartialTextChannel(0),
+                    data=msg_data
+                )
         
         component_data = data.get("data", {})
         component_type = component_data.get("component_type", 1)
